//
//
//  dynarr.pp - part of program UUECoder
//  Author : Andrew V. Sichevoi
//           2:6028/9@fidonet
//           sichevoi@mail.ru
//	     www.sichevoi.nm.ru
//
//  License: GNU GPL (v2.0 or high)
//
//

unit DynArr;

 interface 
 
uses
    Objects; 

//=============================================================================
type
    pStrArr  = ^tStrArr;
     tStrArr  = Array[1..1] of String;
     
    pWordArr = ^tWordArr;
     tWordArr = Array[1..1] of Word;

type
{=[ object.tDynArr ]==========================================================}
    pDynArray = ^tDynArray;
     tDynArray = object
      private
       Index : Word;
       Items : Word;
      public
       function Count : Word;      
     end;
{=[ object.tDynArr ]==========================================================}      

{=[ object.tStrDynArr ]=======================================================}
    pStrDynArray = ^tStrDynArray;
     tStrDynArray = object (tDynArray)
      private
       SArr  : pStrArr;
       _SArr : pStrArr;
      public
       constructor Init;
       procedure   Insert  (Item   : String);
       procedure   InsertAt(aIndex : Word; Item : String);
       procedure   Delete(Num    : Word  );
       function    At    (aIndex : Word) : String;
       destructor  Done;
     end;
{=[ object.tStrDynArr ]=======================================================}     

{=[ object.tWordDynArr ]======================================================}     
    pWordDynArray = ^tWordDynArray;
     tWordDynArray = object (tDynArray)
      private
       WArr  : pWordArr;
       _WArr : pWordArr;
      public
       constructor Init;
       procedure   Insert  (Item : Word);
       procedure   InsertAt(aIndex : Word; Item : Word);
       procedure   Delete(Num  : Word);
       function    At    (aIndex : Word) : Word;
       destructor  Done;
     end;     
{=[ object.tWordDynArr ]======================================================}     
//=============================================================================     
     
  implementation       

//=============================================================================     
{=[ tDynArr.Count ]===========================================================}
function tDynArray.Count : Word;
begin

 Count:= Items;
 
end; 
{=[ tDynArr.Count ]===========================================================}

{*****************************************************************************}

{=[ tStrDynArray.Init ]=======================================================}
constructor tStrDynArray.Init;
begin

 GetMem(SArr , SizeOf(String) * 1);
 GetMem(_SArr, SizeOf(String) * 1);
 Items:= 0;
 
end; 
{=[ tStrDynArray.Init ]=======================================================}

{=[ tStrDynArray.Insert() ]===================================================}
procedure  tStrDynArray.Insert(Item : String);
begin

 for Index:=1 to Items do _SArr^[Index]:= SArr^[Index];
 FreeMem(SArr, SizeOf(String) * Items);
 Inc(Items);
 GetMem(SArr, SizeOf(String) * Items);
 
 for Index:=1 to (Items - 1) do SArr^[Index]:= _SArr^[Index];
 SArr^[Items]:= Item;
 FreeMem(_SArr, SizeOf(String) * (Items - 1));
 GetMem (_SArr, SizeOf(String) * Items);
 
end; 
{=[ tStrDynArray.Insert() ]===================================================}

{=[ tStrDynArray.InsertAt() ]=================================================}
procedure  tStrDynArray.InsertAt(aIndex : Word; Item : String);
begin

  FreeMem(_SArr, SizeOf(String) * Items);
  GetMem (_SArr, SizeOf(String) * (Items + 1));
  for Index:=1 to (aIndex - 1) do _SArr^[Index]:= SArr^[Index];
  _SArr^[aIndex]:= Item; 
  
  for Index:= (aIndex + 1) to (Items + 1) do _SArr^[Index]:= SArr^[Index - 1];
    
  FreeMem(SArr, SizeOf(String) * Items);
  Inc(Items);
  GetMem(SArr, SizeOf(String) * Items);
  for Index:=1 to Items do SArr^[Index]:= _SArr^[Index];

end;
{=[ tStrDynArray.InsertAt() ]=================================================}

{=[ tStrDynArray.Delete() ]===================================================}
procedure  tStrDynArray.Delete(Num : Word);
var
    Cnt : Word;
begin

 for Index:=1 to Num do _SArr^[Index]:= SArr^[Index];
 Cnt:=Index; 
 Inc(Index, 1);
 while (Index <= Items) do begin
  _SArr^[Cnt]:= SArr^[Index];
  Inc(Index);
  Inc(Cnt);
 end;
 
 Dec(Items);
 FreeMem(SArr, SizeOf(String) * (Items + 1));
 GetMem (SArr, SizeOf(String) * Items);
 for Index:=1 to Items do SArr^[Index]:= _SArr^[Index];
 
 FreeMem(_SArr, SizeOf(String) * (Items + 1));
 GetMem (_SArr, SizeOf(String) * Items);

end; 
{=[ tStrDynArray.Delete() ]===================================================}

{=[ tStrDynArray.At() ]=======================================================}
function   tStrDynArray.At(aIndex : Word) : String;
begin

 At:= SArr^[aIndex];
 
end; 
{=[ tStrDynArray.At() ]=======================================================}

{=[ tStrDynArray.Done ]=======================================================}
destructor tStrDynArray.Done;
begin

 FreeMem( SArr, SizeOf(String) * Items);
 FreeMem(_SArr, SizeOf(String) * Items);
 
end; 
{=[ tStrDynArray.Done ]=======================================================}

{*****************************************************************************}

{=[ tWordDynArray.Init ]======================================================}
constructor tWordDynArray.Init;
begin

 GetMem(WArr , SizeOf(Word) * 1);
 GetMem(_WArr, SizeOf(Word) * 1);
 Items:= 0;
 
end; 
{=[ tWordDynArray.Init ]======================================================}

{=[ tWordDynArray.Insert() ]==================================================}
procedure   tWordDynArray.Insert(Item : Word);
begin

 for Index:=1 to Items do _WArr^[Index]:= WArr^[Index];
 FreeMem(WArr, SizeOf(Word) * Items);
 Inc(Items);
 GetMem(WArr, SizeOf(Word) * Items);
 
 for Index:=1 to (Items - 1) do WArr^[Index]:= _WArr^[Index];
 WArr^[Items]:= Item;
 FreeMem(_WArr, SizeOf(Word) * (Items - 1));
 GetMem (_WArr, SizeOf(Word) * Items);

end;
{=[ tWordDynArray.Insert() ]==================================================}

{=[ tWordDynArray.InsertAt() ]================================================}
procedure  tWordDynArray.InsertAt(aIndex : Word; Item : Word);
begin

  FreeMem(_WArr, SizeOf(Word) * Items);
  GetMem (_WArr, SizeOf(Word) * (Items + 1));
  for Index:=1 to (aIndex - 1) do _WArr^[Index]:= WArr^[Index];
  _WArr^[aIndex]:= Item; 
  
  for Index:= (aIndex + 1) to (Items + 1) do _WArr^[Index]:= WArr^[Index - 1];
    
  FreeMem(WArr, SizeOf(Word) * Items);
  Inc(Items);
  GetMem(WArr, SizeOf(Word) * Items);
  for Index:=1 to Items do WArr^[Index]:= _WArr^[Index];

end;
{=[ tWordDynArray.InsertAt() ]================================================}

{=[ tWordDynArray.Delete() ]==================================================}
procedure   tWordDynArray.Delete(Num : Word);
var
    Cnt : Word;
begin

 for Index:=1 to Num do _WArr^[Index]:= WArr^[Index];
 Cnt:=Index; 
 Inc(Index, 1);
 while (Index <= Items) do begin
  _WArr^[Cnt]:= WArr^[Index];
  Inc(Index);
  Inc(Cnt);
 end;
 
 Dec(Items);
 FreeMem(WArr, SizeOf(Word) * (Items + 1));
 GetMem (WArr, SizeOf(Word) * Items);
 for Index:=1 to Items do WArr^[Index]:= _WArr^[Index];
 
 FreeMem(_WArr, SizeOf(Word) * (Items + 1));
 GetMem (_WArr, SizeOf(Word) * Items);
 
end; 
{=[ tWordDynArray.Delete() ]==================================================}

{=[ tWordDynArray.At() ]======================================================}
function   tWordDynArray.At(aIndex : Word) : Word;
begin

 At:= WArr^[aIndex];
 
end; 
{=[ tWordDynArray.At() ]======================================================}

{=[ tWordDynArray.Done ]======================================================}
destructor tWordDynArray.Done;
begin

 FreeMem( WArr, SizeOf(Word) * Items);
 FreeMem(_WArr, SizeOf(Word) * Items);
 
end; 
{=[ tWordDynArray.Done ]======================================================}

//=============================================================================

end.