//
//
//  ucparam.pp - part of program UUECoder
//  Author : Andrew V. Sichevoi
//           2:6028/9@fidonet
//           sichevoi@mail.ru
//	     www.sichevoi.nm.ru
//
//  License: GNU GPL (v2.0 or high)
//
//

unit UCparam;

 interface

//=============================================================================

    function  GetParamStr : String;
    procedure ParseParamStr(Str : String);
    procedure CheckParams;
    procedure ParseStr(var InStr : String);
    
type
    tMailType = (None, NetMail, EchoMail);    
    
const
    //
     _ParamStr : String = '';

     _Config   : String = ''; { Config          }
     _FName    : String = ''; { From name       }
     _FAddr    : String = ''; { From address    }
     _TName    : String = ''; { To name         }
     _TAddr    : String = ''; { To address      }
     _Subject  : String = ''; { Subject         }
     _TAGLine  : String = ''; { TAGLine         }
     _TearLine : String = ''; { TearLine        }
     _Origin   : String = ''; { Origin          }
     _HeadTPL  : String = ''; { Header template }
     _FootTPL  : String = ''; { Footer template }
     _FileName : String = ''; { Filename        }
     _EchoPath : String = ''; { Path to echo    }
     _MailType : tMailType = None; { Type of mail: NetMail or EchoMail}
     _Lines    : Word   = 50;
     _Attrib   : String = '';
     
     aLocal     : Boolean = true;
     aKillSnt   : Boolean = false;
     aPvt       : Boolean = true;
     aHold      : Boolean = false;
{     aImm       : Boolean = false;
}
     aCrash     : Boolean = false;
     aFileF     : Boolean = false;
{     aErasSnt   : Boolean = false;
}
     aFreq      : Boolean = false;
     aRRQ       : Boolean = false;
{     aDir       : Boolean = false;
}
     aARQ       : Boolean = false;
     aSnt       : Boolean = false;
     aURQ       : Boolean = false;
     aTransit   : Boolean = false;
     
     

     Kav      = '"';

//=============================================================================    

  implementation
 
uses
    UCStr,
    UCCommon,
    UCVersion,
    FidoMask,
    SysUtils;  
    
{=[ GetParamStr ]=============================================================}
function GetParamStr : String;
begin

 GetParamStr:= CmdLine;
 
end; 
{=[ GetParamStr ]=============================================================}

{=[ Flag ]====================================================================}
function Flag(C : Char) : Boolean;
begin
 
 Flag:= false;
 if (Pos(C, UpStr(_Attrib)) <> 0) then Flag:= true;

end;
{=[ Flag ]====================================================================}


{=[ ParseParamStr ]===========================================================}
procedure ParseParamStr(Str : String);
var
   Ps   : Byte;
   I    : Byte;
   Temp : String;

{===}
function PsCopy : String;
var
   S : String;
begin

 I:=Ps+2;
 S:='';
  while (Str[I] <> ' ') and (I <> Length(Str)+1) do begin
   S:=S+Str[I];
   Inc(I);
  end;
 PsCopy:=S;
 
end;

{===}
function PsCopy2 : String;
begin
 if Str[Ps+2] = Kav then PsCopy2:=CutWord(Copy(Str, Ps, $FF), Kav, Kav) else
                         PsCopy2:=PsCopy;
end;

{===}
begin

 Ps:=Pos('-c', Str);             
 if (Ps <> 0) then _Config:=PsCopy2;    { config }

 Ps:=Pos('-o', Str);      
 if (Ps <> 0) then _FAddr :=PsCopy;
     { From address }

 Ps:=Pos('-f', Str);      
 if (Ps <> 0) then _FName :=PsCopy2;    { From name }

 Ps:=Pos('-s', Str);                  
 if (Ps <> 0) then _Subject:=PsCopy2;
   { Subject }

 Ps:=Pos('-d', Str);      
 if (Ps <> 0) then _TAddr :=PsCopy;
     { To address }

 Ps:=Pos('-t', Str);      
 if (Ps <> 0) then _TName :=PsCopy2;
    { To name }

 Ps:=Pos('=T', Str);
 if (Ps <> 0) then _TAGLine :=PsCopy2;  { TAGLine }

 Ps:=Pos('+T', Str);                   
 if (Ps <> 0) then _TearLine:=PsCopy2;
  { TearLine }

 Ps:=Pos('+O', Str);
 if (Ps <> 0) then _Origin  :=PsCopy2;
  { Origin }

 Ps:=Pos('+h', Str);
 if (Ps <> 0) then _HeadTPL :=PsCopy2;
  { Header template }

 Ps:=Pos('+f', Str);
 if (Ps <> 0) then _FootTPL :=PsCopy2;
  { Footer template }

 Ps:=Pos('-F', Str);
 if (Ps <> 0) then _FileName:=PsCopy2;
  { FileName }

 Ps:=Pos('-e', Str);
 if (Ps <> 0) then _EchoPath:=PsCopy2;  { EchoName }
 
 Ps:=Pos('-m', Str);                    { Type of mail }
 if (Ps <> 0) then begin
  Temp:=PsCopy2;
  if (UpStr(Temp) = 'E') then _MailType:=EchoMail else
  if (UpStr(Temp) = 'N') then _MailType:=NetMail;
 end;
 
 Ps:=Pos('-l', Str);
 if (Ps <> 0) then _Lines:=Round(SVal(PsCopy)); { Lines }

 Ps:=Pos('-a', Str);
 if (Ps <> 0) then _Attrib:= PsCopy;

end;

{=[ ParseParamStr ]===========================================================}
{=[ CheckParams ]=============================================================}
procedure CheckParams;
var
   F : Text;
   S : String;
begin

//   [start]
 if (_Config <> '') then begin                 {     }
  if (not FileExists(_Config)) then 
   Error('Can''t open "'+_Config+'" !') else begin
    Assign(F, _Config);
    Reset(F);
    
    while (not EOF(F)) do begin
     Readln(F, S);
     if (S = '') then continue;
     if ((S[1] = CSym) or (S = CSym)) then continue;
     ParseParamStr(S);
    end;
    
    Close(F);
  end; 
 end;
 ParseParamStr(_ParamStr);               { :  ->  }
//   [end]

//     [start]
 if (_FAddr = '')            then Error('You must define orig. fido address!');
 if (not IsFidoAddr(_FAddr)) then Error('Orig. fido address is not correct!');
 if (_FName = '')            then _FName:='UUECoder';
 
 if (_MailType = None)       then Error('You must switch type of mail!');
{===} 
 if ((_TAddr = '') and (_MailType = NetMail)) then
                                  Error('You must define dest. address!');
 if (not IsFidoAddr(_TAddr)) then Error('Dest. fido address is not correct!');
 if (_TName = '') then begin
  if (_MailType = NetMail)  then _TName:='SysOp' else
  if (_MailType = EchoMail) then _TName:='All';
 end;
{===}
 if (_TearLine = '') then _TearLine:= ProgramFullPID;
 if (_Origin   = '') then _Origin  := ProgramName + Space + ProgramVersion;			      
{===}
 if (_HeadTPL = '') then Error('You must define header template!');
 if (_FootTPL = '') then Error('You must define footer template!');
 
 if (not FileExists(_HeadTPL)) then Error('Can''t open "'+_HeadTPL+'" !');
 if (not FileExists(_FootTPL)) then Error('Can''t open "'+_FootTPL+'" !');
{===}
 if (_EchoPath = '') then Error('You must define path to echo!');
{===}
 if (_Lines = 0) then Error('Uncorrect num of lines!');
{===}
 if (_FileName = '') then Error('FileName must be defined!');
 if (not FileExists(_FileName) ) then Error('Can''t open "'+_FileName+'" !');
{===}
 if (_Attrib <> '') then begin
  aLocal  := Flag('L');
  aKillSnt:= Flag('K');
  aPvt    := Flag('K');
  aHold   := Flag('H');
  aCrash  := Flag('C');
  aFileF  := Flag('F');
  aFreq   := Flag('R');
  aRRQ    := Flag('Q');
  aARQ    := Flag('A');
  aSnt    := Flag('S');
  aURQ    := Flag('U');
  aTransit:= Flag('T');
 end; 
//     [end]

end;
{=[ CheckParams ]=============================================================}

{=[ ParseStr ]================================================================}
procedure ParseStr(var InStr : String);
begin

 InStr:= MacroReplace(InStr, '@FName'   , _FName);
 InStr:= MacroReplace(InStr, '@TName'   , _TName);
 InStr:= MacroReplace(InStr, '@FAddr'   , _FAddr);
 InStr:= MacroReplace(InStr, '@TAddr'   , _TAddr);
 InStr:= MacroReplace(InStr, '@Subject' , _Subject);
 InStr:= MacroReplace(InStr, '@FileName', _FileName);
 
 InStr:= MacroReplace(InStr, '@K'       , #1);
 InStr:= MacroReplace(InStr, '@PName'   , ProgramName);
 InStr:= MacroReplace(InStr, '@PVer'    , ProgramVersion);
 InStr:= MacroReplace(InStr, '@POSS'    , SPlatform);
 InStr:= MacroReplace(InStr, '@POSF'    , FPlatform);
 InStr:= MacroReplace(InStr, '@PID'     , ProgramFullPID);
 
end;
{=[ ParseStr ]================================================================}

end.